@@ -1,6 +1,7 @@
 from .core import *
-from .relationship import (NotConnected, OUTGOING, INCOMING, EITHER,
+from .relationship_manager import (NotConnected, OUTGOING, INCOMING, EITHER,
         RelationshipTo, RelationshipFrom, Relationship)
+from .relationship import StructuredRel
 from .cardinality import (AttemptedCardinalityViolation,
         CardinalityViolation, ZeroOrMore, OneOrMore, ZeroOrOne, One)
 from .properties import (StringProperty, IntegerProperty, AliasProperty,
