@@ -1,20 +1,20 @@
 from py2neo import neo4j, cypher
-from .properties import Property, AliasProperty
-from .relationship import RelationshipManager, OUTGOING, RelationshipDefinition
-from .exception import (DoesNotExist, RequiredProperty, CypherException,
-        NoSuchProperty)
+from .exception import DoesNotExist, CypherException
 from .util import camel_to_upper, CustomBatch, _legacy_conflict_check
+from .properties import Property, PropertyManager, AliasProperty
+from .relationship_manager import RelationshipManager, OUTGOING
 from .traversal import TraversalSet
-import types
 from .signals import hooks
 from .index import NodeIndexManager
+import logging
 import os
 import sys
+logger = logging.getLogger(__name__)
 
 if sys.version_info >= (3, 0):
     from urllib.parse import urlparse
 else:
-    from urlparse import urlparse
+    from urlparse import urlparse # noqa
 
 
 DATABASE_URL = os.environ.get('NEO4J_REST_URL', 'http://localhost:7474/db/data/')
@@ -37,6 +37,9 @@ def connection():
 
 
 def cypher_query(query, params=None):
+    if os.environ.get('NEOMODEL_CYPHER_DEBUG', False):
+        logger.debug(query)
+        logger.debug("params: " + repr(params))
     try:
         return cypher.execute(connection(), query, params)
     except cypher.CypherError as e:
@@ -50,6 +53,7 @@ class CypherMixin(object):
         return connection()
 
     def cypher(self, query, params=None):
+        self._pre_action_check('cypher')
         assert hasattr(self, '__node__')
         params = params or {}
         params.update({'self': self.__node__.id})
@@ -57,7 +61,6 @@ class CypherMixin(object):
 
 
 class StructuredNodeMeta(type):
-
     def __new__(mcs, name, bases, dct):
         dct.update({'DoesNotExist': type('DoesNotExist', (DoesNotExist,), dct)})
         inst = super(StructuredNodeMeta, mcs).__new__(mcs, name, bases, dct)
@@ -72,10 +75,13 @@ class StructuredNodeMeta(type):
                     # support for 'magic' properties
                     if hasattr(value, 'setup') and hasattr(value.setup, '__call__'):
                         value.setup()
+            if '__index__' in dct:
+                name = dct['__index__']
             inst.index = NodeIndexManager(inst, name)
         return inst
 
-StructuredNodeBase = StructuredNodeMeta('StructuredNodeBase', (object, ), {})
+
+StructuredNodeBase = StructuredNodeMeta('StructuredNodeBase', (PropertyManager,), {})
 
 
 class StructuredNode(StructuredNodeBase, CypherMixin):
@@ -86,30 +92,14 @@ class StructuredNode(StructuredNodeBase, CypherMixin):
 
     __abstract_node__ = True
 
+    def __init__(self, *args, **kwargs):
+        self.__node__ = None
+        super(StructuredNode, self).__init__(*args, **kwargs)
+
     @classmethod
     def category(cls):
         return category_factory(cls)
 
-    def __init__(self, *args, **kwargs):
-        try:
-            super(StructuredNode, self).__init__(*args, **kwargs)
-        except TypeError:
-            super(StructuredNode, self).__init__()
-        self.__node__ = None
-        for key, val in self._class_properties().items():
-            if val.__class__ is RelationshipDefinition:
-                self.__dict__[key] = val.build_manager(self, key)
-            # handle default values
-            elif issubclass(val.__class__, Property)\
-                    and not isinstance(val, AliasProperty)\
-                    and not issubclass(val.__class__, AliasProperty):
-                if not key in kwargs or kwargs[key] is None:
-                    if val.has_default:
-                        kwargs[key] = val.default_value()
-        for key, value in kwargs.items():
-            if not(key.startswith("__") and key.endswith("__")):
-                setattr(self, key, value)
-
     def __eq__(self, other):
         if not isinstance(other, (StructuredNode,)):
             raise TypeError("Cannot compare neomodel node with a " + other.__class__.__name__)
@@ -120,18 +110,6 @@ class StructuredNode(StructuredNodeBase, CypherMixin):
             raise TypeError("Cannot compare neomodel node with a " + other.__class__.__name__)
         return self.__node__ != other.__node__
 
-    @property
-    def __properties__(self):
-        node_props = {}
-        for key, value in super(StructuredNode, self).__dict__.items():
-            if (not key.startswith('_')
-                    and not isinstance(value, types.MethodType)
-                    and not isinstance(value, RelationshipManager)
-                    and not isinstance(value, AliasProperty)
-                    and value is not None):
-                node_props[key] = value
-        return node_props
-
     @hooks
     def save(self):
         # create or update instance node
@@ -142,26 +120,35 @@ class StructuredNode(StructuredNodeBase, CypherMixin):
             batch.set_properties(self.__node__, props)
             self._update_indexes(self.__node__, props, batch)
             batch.submit()
+        elif hasattr(self, '_is_deleted') and self._is_deleted:
+            raise ValueError("{}.save() attempted on deleted node".format(self.__class__.__name__))
         else:
             self.__node__ = self.create(self.__properties__)[0].__node__
             if hasattr(self, 'post_create'):
                 self.post_create()
         return self
 
+    def _pre_action_check(self, action):
+        if hasattr(self, '_is_deleted') and self._is_deleted:
+            raise ValueError("{}.{}() attempted on deleted node".format(self.__class__.__name__, action))
+        if self.__node__ is None:
+            raise ValueError("{}.{}() attempted on unsaved node".format(self.__class__.__name__, action))
+
     @hooks
     def delete(self):
-        if self.__node__:
-            self.index.__index__.remove(entity=self.__node__)
-            self.cypher("START self=node({self}) MATCH (self)-[r]-() DELETE r, self")
-            self.__node__ = None
-        else:
-            raise Exception("Node has not been saved so cannot be deleted")
+        self._pre_action_check('delete')
+        self.index.__index__.remove(entity=self.__node__)
+        self.cypher("START self=node({self}) MATCH (self)-[r]-() DELETE r, self")
+        self.__node__ = None
+        self._is_deleted = True
         return True
 
-    def traverse(self, rel_manager):
-        return TraversalSet(self).traverse(rel_manager)
+    def traverse(self, rel_manager, *args):
+        self._pre_action_check('traverse')
+        return TraversalSet(self).traverse(rel_manager, *args)
 
     def refresh(self):
+        self._pre_action_check('refresh')
         """Reload this object from its node in the database"""
         if self.__node__:
             if self.__node__.exists():
@@ -196,7 +183,7 @@ class StructuredNode(StructuredNodeBase, CypherMixin):
             if (issubclass(prop.__class__, Property)
                     and not isinstance(prop, AliasProperty)):
                 if key in node.__metadata__['data']:
-                    props[key] = prop.inflate(node.__metadata__['data'][key], node_id=node.id)
+                    props[key] = prop.inflate(node.__metadata__['data'][key], node)
                 elif prop.has_default:
                     props[key] = prop.default_value()
                 else:
@@ -206,46 +193,10 @@ class StructuredNode(StructuredNodeBase, CypherMixin):
         snode.__node__ = node
         return snode
 
-    @classmethod
-    def deflate(cls, node_props, node_id=None):
-        """ deflate dict ready to be stored """
-        deflated = {}
-        for key, prop in cls._class_properties().items():
-            if (not isinstance(prop, AliasProperty)
-                    and issubclass(prop.__class__, Property)):
-                if key in node_props and node_props[key] is not None:
-                    deflated[key] = prop.deflate(node_props[key], node_id=node_id)
-                elif prop.has_default:
-                    deflated[key] = prop.deflate(prop.default_value(), node_id=node_id)
-                elif prop.required:
-                    raise RequiredProperty(key, cls)
-        return deflated
-
     @classmethod
     def relationship_type(cls):
         return camel_to_upper(cls.__name__)
 
-    @classmethod
-    def get_property(cls, name):
-        try:
-            node_property = getattr(cls, name)
-        except AttributeError:
-            raise NoSuchProperty(name, cls)
-        if not issubclass(node_property.__class__, Property)\
-                or not issubclass(node_property.__class__, AliasProperty):
-            NoSuchProperty(name, cls)
-        return node_property
-
-    @classmethod
-    def _class_properties(cls):
-        # get all dict values for inherited classes
-        # reverse is done to keep inheritance order
-        props = {}
-        for scls in reversed(cls.mro()):
-            for key, value in scls.__dict__.items():
-                props[key] = value
-        return props
-
     @classmethod
     def _update_indexes(cls, node, props, batch):
         # check for conflicts prior to execution
@@ -272,6 +223,9 @@ class CategoryNode(CypherMixin):
     def traverse(self, rel):
         return TraversalSet(self).traverse(rel)
 
+    def _pre_action_check(self, action):
+        pass
+
 
 class InstanceManager(RelationshipManager):
     """Manage 'instance' rel of category nodes"""
