@@ -1,8 +1,31 @@
-from .relationship import RelationshipDefinition, rel_helper, INCOMING
+from .relationship_manager import RelationshipDefinition, rel_helper, INCOMING
 from copy import deepcopy
 import re
 
 
+def _deflate_node_value(target_map, prop, value):
+    prop = prop.replace('!', '').replace('?', '')
+    property_classes = set()
+    # find properties on target classes
+    for node_cls in target_map.values():
+        if hasattr(node_cls, prop):
+            property_classes.add(getattr(node_cls, prop).__class__)
+
+    # attempt to deflate
+    if len(property_classes) == 1:
+        return property_classes.pop()().deflate(value)
+    elif len(property_classes) > 1:
+        classes = ' or '.join([cls.__name__ for cls in property_classes])
+        node_classes = ' or '.join([cls.__name__ for cls in target_map.values()])
+        raise ValueError("Unsure how to deflate '" + value + "' conflicting definitions "
+                + " for target node classes " + node_classes + ", property could be any of: "
+                + classes + " in where()")
+    else:
+        node_classes = ', '.join([cls.__name__ for cls in target_map.values()])
+        raise ValueError("No property '{}' on {} can't deflate '{}' for where()".format(
+            prop, node_classes, value))
+
+
 def last_x_in_ast(ast, x):
     assert isinstance(ast, (list,))
     for node in reversed(ast):
@@ -31,14 +54,17 @@ class AstBuilder(object):
             'class': self.start_node.__class__, 'name': 'origin'}]
         self.origin_is_category = start_node.__class__.__name__ == 'CategoryNode'
 
-    def _traverse(self, rel_manager):
+    def _traverse(self, rel_manager, where_stmts=None):
         if len(self.ast) > 1:
             t = self._find_map(self.ast[-2]['target_map'], rel_manager)
         else:
             t = getattr(self.start_node, rel_manager).definition
             t['name'] = rel_manager
 
-        match, where = self._build_match_ast(t)
+        if where_stmts and not 'model' in t:
+                raise Exception("Conditions " + repr(where_stmts) + " to traverse "
+                        + rel_manager + " not allowed as no model specified on " + rel_manager)
+        match, where = self._build_match_ast(t, where_stmts)
         self._add_match(match)
         if where:
             self._add_where(where)
@@ -68,7 +94,7 @@ class AstBuilder(object):
         self.ident_count += 1
         return 'r' + str(self.ident_count)
 
-    def _build_match_ast(self, target):
+    def _build_match_ast(self, target, where_stmts):
         rel_to_traverse = {
             'lhs': last_x_in_ast(self.ast, 'name')['name'],
             'direction': target['direction'],
@@ -83,6 +109,10 @@ class AstBuilder(object):
             'target_map': target['target_map']
         }
 
+        where_clause = []
+        if where_stmts:
+            where_clause = self._where_rel(where_stmts, rel_to_traverse['ident'], target['model'])
+
         # if we aren't category node or already traversed one rel
         if not self.origin_is_category or len(self.ast) > 1:
             category_rel_ident = self._create_ident()
@@ -94,9 +124,9 @@ class AstBuilder(object):
                 'rhs': ''
             })
             # Add where
-            return match, [category_rel_ident + '.__instance__! = true']
+            where_clause.append(category_rel_ident + '.__instance__! = true')
 
-        return match, []
+        return match, where_clause
 
     def _find_map(self, target_map, rel_manager):
         targets = []
@@ -118,19 +148,37 @@ class AstBuilder(object):
         # return as list if more than one
         return targets if len(targets) > 1 else targets[0]
 
-    def _add_filter(self, ident_prop, op, value):
+    def _where_node(self, ident_prop, op, value):
         if re.search(r'[^\w\?\!\.]', ident_prop):
             raise Exception("Invalid characters in ident allowed: [. \w ! ?]")
+        target = last_x_in_ast(self.ast, 'name')
         if not '.' in ident_prop:
-            ident_prop = last_x_in_ast(self.ast, 'name')['name'] + '.' + ident_prop
+            prop = ident_prop
+            ident_prop = target['name'] + '.' + ident_prop
+        else:
+            prop = ident_prop.split('.')[1]
+        value = _deflate_node_value(target['target_map'], prop, value)
+        return self._where_expr(ident_prop, op, value)
 
+    def _where_rel(self, statements, rel_ident, model):
+        stmts = []
+        for statement in statements:
+            rel_prop = statement[0].replace('!', '').replace('?', '')
+            prop = getattr(model, rel_prop)
+            if not prop:
+                raise AttributeError("RelationshipManager '{}' on {} doesn't have a property '{}' defined".format(
+                    rel_ident, self.start_node.__class__.__name__, rel_prop))
+            val = prop.__class__().deflate(statement[2])
+            stmts.append(self._where_expr(rel_ident + "." + statement[0], statement[1], val))
+        return stmts
+
+    def _where_expr(self, ident_prop, op, value):
         if not op in ['>', '<', '=', '<>', '=~']:
             raise Exception("Operator not supported: " + op)
-
         placeholder = re.sub('[!?]', '', ident_prop.replace('.', '_'))
         placeholder = unique_placeholder(placeholder, self.query_params)
         self.query_params[placeholder] = value
-        self._add_where([" ".join([ident_prop, op, '{' + placeholder + '}'])])
+        return " ".join([ident_prop, op, '{' + placeholder + '}'])
 
     def _add_return(self, ast):
         node = last_x_in_ast(ast, 'name')
@@ -143,6 +191,15 @@ class AstBuilder(object):
         if hasattr(self, '_limit'):
             ast.append({'limit': int(self._limit)})
 
+    def _add_return_rels(self, ast):
+        node = last_x_in_ast(ast, 'name')
+        idents = [node['match'][0]['ident']]
+        ast.append({'return': idents})
+        if hasattr(self, '_skip'):
+            ast.append({'skip': int(self._skip)})
+        if hasattr(self, '_limit'):
+            ast.append({'limit': int(self._limit)})
+
     def _set_order(self, ident_prop, desc=False):
         if not '.' in ident_prop:
             ident_prop = last_x_in_ast(self.ast, 'name')['name'] + '.' + ident_prop
@@ -161,6 +218,8 @@ class AstBuilder(object):
             raise NotImplemented("Order already set")
 
     def _add_return_count(self, ast):
+        if hasattr(self, '_skip') or hasattr(self, '_limit'):
+            raise NotImplemented("Can't use skip or limit with count")
         node = last_x_in_ast(ast, 'name')
         ident = ['count(' + node['name'] + ')']
         node = last_x_in_ast(ast, 'name')
@@ -177,7 +236,7 @@ class AstBuilder(object):
         self.last_ast = ast
         return results
 
-    def execute_and_inflate(self, ast):
+    def execute_and_inflate_nodes(self, ast):
         target_map = last_x_in_ast(ast, 'target_map')['target_map']
         results = self.execute(ast)
         nodes = [row[0] for row in results]
@@ -190,10 +249,10 @@ class TraversalSet(AstBuilder):
     def __init__(self, start_node):
         super(TraversalSet, self).__init__(start_node)
 
-    def traverse(self, rel):
+    def traverse(self, rel, *where_stmts):
         if not self.start_node.__node__:
             raise Exception("Cannot traverse unsaved node")
-        self._traverse(rel)
+        self._traverse(rel, where_stmts)
         return self
 
     def order_by(self, prop):
@@ -205,7 +264,8 @@ class TraversalSet(AstBuilder):
         return self
 
     def where(self, ident, op, value):
-        self._add_filter(ident, op, value)
+        expr = self._where_node(ident, op, value)
+        self._add_where([expr])
         return self
 
     def skip(self, count):
@@ -223,7 +283,7 @@ class TraversalSet(AstBuilder):
     def run(self):
         ast = deepcopy(self.ast)
         self._add_return(ast)
-        return self.execute_and_inflate(ast)
+        return self.execute_and_inflate_nodes(ast)
 
     def __iter__(self):
         return iter(self.run())
