@@ -1,4 +1,5 @@
 from neomodel import StructuredNode, StringProperty, IntegerProperty, UniqueProperty
+from lucenequerybuilder import Q
 
 
 class Human(StructuredNode):
@@ -32,14 +33,11 @@ def test_optional_properties_dont_get_indexed():
 
 
 def test_lucene_query():
-    try:
-        from lucenequerybuilder import Q
-    except ImportError:
-        return
     Human(name='sarah', age=3).save()
     Human(name='jim', age=4).save()
     Human(name='bob', age=5).save()
     Human(name='tim', age=2).save()
+
     names = [p.name for p in Human.index.search(Q('age', inrange=[3, 5]))]
     assert 'sarah' in names
     assert 'jim' in names
@@ -91,3 +89,19 @@ def test_index_inherited_props():
     assert MixedHuman.index.name == 'MixedHuman'
     node = MixedHuman.index.get(extra='extra')
     assert node.name == jim.name
+
+
+def test_custom_index_name():
+    class Giraffe(StructuredNode):
+        __index__ = 'GiraffeIndex'
+        name = StringProperty(unique_index=True)
+
+    jim = Giraffe(name='timothy').save()
+    assert Giraffe.index.name == 'GiraffeIndex'
+    node = Giraffe.index.get(name='timothy')
+    assert node.name == jim.name
+
+    class SpecialGiraffe(Giraffe):
+        power = StringProperty()
+
+    assert SpecialGiraffe.index.name != 'GiraffeIndex'
